/*
    File:       nmea.h
    Version:    0.2.0
    Date:       Feb. 23, 2013 - Jul. 07, 2020
	License:	GPL v2
    
	NMEA GPS content parser
    
    ****************************************************************************
    Copyright (C) 2013 Radu Motisan  <radu.motisan@gmail.com>
	
	http://www.pocketmagic.net

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ****************************************************************************
 */
#pragma once

#include <stdint.h>

#define MAX_BUFFER 100
#define MAX_WORD 15
#define MIN(x,y) (x<y)?x:y


class NMEA {
		uint8_t buf[MAX_BUFFER];
		uint8_t bufIndex = 0, starIndex = 0;
		char word[MAX_WORD] = {0};

		// results
		uint8_t hour, min, sec,				// UTC Time
				fix, 						// GPS quality indicator (0=invalid; 1=GPS fix; 2=Diff. GPS fix)
				sats,						// sats in view
				day, month, year,
				checksum,
				ready;
		double latitude, longitude, altitude, speed;

		// aux functions
		uint8_t			digit2dec(char hexdigit);
		double			string2double(char* s);
		uint8_t 		hexdigit(uint8_t digit);
		uint32_t 		hex2int(char *hex);
		double 			decodeCoordinate(double value);

		// string functions
		uint8_t 		mstrncmp( char* s1,  char* s2, uint8_t n);
		char* 			mstrncpy(char *dest,  char *src, uint8_t n);

		bool 			parsedata(uint8_t messageType);
	public:
		// constructor, initing a few variables
		NMEA() {

		}
		
		/*
		 * The serial data is assembled on the fly, without using any redundant buffers. 
		 * When a sentence is complete (one that starts with $, ending in EOL), all processing is done on 
		 * this temporary buffer that we've built: checksum computation, extracting sentence "words" (the CSV values), 
		 * and so on.
		 * When a new sentence is fully assembled using the fusedata function, the code calls parsedata. 
		 * This function in turn, splits the sentences and interprets the data. Here is part of the parser function, 
		 * handling both the $GPRMC NMEA sentence:
		 */
		uint8_t			fusedata(char c);
		

		// READER functions: retrieving results, call isdataready() first
		bool			isdataready();
		uint8_t			getHour();
		uint8_t			getMinute();
		uint8_t			getSecond();
		uint8_t			getDay();
		uint8_t			getMonth();
		uint8_t			getYear();
		double			getLatitude();
		double			getLongitude();
		uint8_t			getSatellites();
		double			getAltitude();
		double			getSpeed();

	};
